program amsnplaying;

uses
	baseunix,
	Classes,
	IniFiles,
	process,
	StrUtils,
	sysutils,
	
	blcksock,
	synacode;
	
var
	host, email, password: String;
	artist, title, album: String;

procedure Log(buffer: String);
var
	logfile: text;
begin
	// To enable logging, uncomment the "exit" line and change the log file path.
	exit;
	Assign(logfile, '/home/daniel/Desktop/amsnplaying.log');
	Append(logfile);
	writeLn(logfile, buffer);
	Close(logfile);
	// Also write it to stdout
	WriteLn(buffer);
end;

// Get data from Amarok, via DCOP
function getData(what: String): String;
var
	aProcess: TProcess;
	temp: TStringList;
begin
	aProcess := TProcess.Create(nil);
	aProcess.CommandLine := 'dcop amarok player ' + what;
	// Wait for process to end, and use stdout pipe.
	aProcess.Options := AProcess.Options + [poWaitOnExit, poUsePipes];
	aProcess.Execute;
	// Read output into temp string list
	temp := TStringList.Create;
	temp.LoadFromStream(aProcess.Output);
	// Return the first string.
	Result := temp[0];

	temp.Free;
	aProcess.Free;
end;

// Pop up a message in Amarok
procedure AmarokPopup(message: String);
var
	aProcess: TProcess;
begin
	aProcess := TProcess.Create(nil);
	aProcess.CommandLine := 'dcop amarok playlist popupMessage "' + message + '"';
	aProcess.Execute;
	// Also write it to stdout.
	writeLn(message);
end;

// Get the HMAC-MD5 hash of a string.
function HMACStr(text: String; key: String): String;
var
	md5: String = '';
	i: Integer;
begin
	Result := '';
	
	// Get our hash.
	md5 := HMAC_MD5(text, key);
	// Convert it to hex representation
	for i := 1 to Length(md5) do
	begin
		Result := Result + IntToHex(Ord(md5[i]), 2);
		//Result := Result + IntToHex(md5[i], 2);
	end;
	// Make it lowercase
	Result := lowercase(Result);
end;

// Update media in aMSN
procedure UpdateaMSN(enabled: Boolean; media: String);
var
	sock: TTCPBlockSocket;
	temp: String;
begin
	// Create a socket.
	sock := TTCPBlockSocket.Create;
	// Connect to the host we want.
	sock.Connect(host, '63251');
	if sock.LastError <> 0 then
	begin
		AmarokPopup('Could not connect to aMSN remote control! ' + sock.GetErrorDescEx);
		halt(2);
	end;
	// Write our account name to the socket.
	sock.SendString(email + #10);
	// Read the input (should be the port number).
	temp := sock.RecvString(5000);
	// Close this connection.
	sock.CloseSocket();
	// Connect to the new port.
	Log('This account is on port ' + temp);
	sock.Connect(host, temp);
	// Start authenticating.
	sock.SendString('auth'#10);
	temp := sock.RecvString(5000);
	// TODO: This is a bit of a hack, but works for now :P
	// Our key for the auth
	temp := ExtractWord(2, temp, [' ', '}']);
	// Send our authentication.
	Log('Received auth key: ' + temp);
	temp := HMACStr(password, temp);
	Log('Sending auth: ' + temp);
	sock.SendString('auth2 ' + temp + #10);
	// See what it says about this.
	temp := sock.RecvString(5000);
	// Did it not work?
	if temp <> '{Authentication successfull} normal' then
	begin
		AmarokPopup('Error authenticating to aMSN. Are you sure you set your password correctly? aMSN returned: ' + temp);
		halt(3);
	end;
	
	// Send the media string.
	if enabled = False then
		sock.SendString('setmedia Music 0'#10)
	else
		sock.SendString('setmedia Music 1 ' + media + #10);
	// See what aMSN says about it.
	//temp := sock.RecvString(5000);
	//writeLn(temp);
	sock.CloseSocket();
end;

procedure UpdateData();
begin
	// Get our data.
	artist := GetData('artist');
	title := GetData('title');
	album := GetData('album');
	
	Log('Artist: ' + artist + ', title: ' + title + ', album: ' + album);
	// Now, we change the media in aMSN.
	if title = '' then
		UpdateaMSN(False, '')
	else
	begin
		// Do we have an album name?
		if album <> '' then
			UpdateaMSN(True, artist + ' - ' + title + ' (Album: ' + album + ')')
		else
			UpdateaMSN(True, artist + ' - ' + title);
	end;
end;

// Clean up on quit.
procedure Quit();
begin
	Log('Quitting!');
	// Remove personal message in aMSN.
	UpdateaMSN(False, '');
	// Actually quit.
	halt(0);
end;

procedure Main();
var
	config: TINIFile;
	buffer: String;
	sigterm_handler: SignalHandler;
begin
	// Read in our config.
	config := TINIFile.Create('amsnplaying.ini');
	host := config.ReadString('Settings', 'host', 'localhost');
	email := config.ReadString('Settings', 'email', '');
	password := config.ReadString('Settings', 'password', '');
	// Make sure we're configured.

	if password = '' then
	begin
		AmarokPopup('Please configure this script by editing amsnplaying.ini first.');
		halt(1);
	end;
	
	// Install our signal handler for when we quit.
	sigterm_handler := SignalHandler(Quit);
	FpSignal(SIGTERM, sigterm_handler);

	// Do an initial update.
	UpdateData();
	
	// This is our main loop, we keep reading the stdin
	while true do
	begin
		// Read the stdin.
		readLn(buffer);
		Log('----');
		Log('Received: ' + buffer);	
		
		// Did we start playing?
		if buffer = 'engineStateChange: playing' then
			UpdateData()
		// If it's paused or stopped, hide it.
		else if (buffer = 'engineStateChange: paused') or (buffer = 'engineStateChange: empty') or (buffer = 'engineStateChange: idle') then
			UpdateaMSN(False, '');
	end;
end;

begin
	Log('Started');
	Main();
end.
